unit Maprot;
{ This is the 32bit version of the TMapRot control by Daniel Parnell

  Email  daniel@comsense.com
  
}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  PBitmapData=^TBitmapData;
  TBitmapData=Array[0..199,0..319] of Byte;

type
  TPic = array[0..127,0..127] of byte;

  TMaprot = class(TGraphicControl)
  private
    procedure paint; override;
  protected
    FPalette:HPalette;

    procedure makepalette; virtual;
    procedure killpalette; virtual;
    function getpalette:hpalette; override;

    procedure MapLine(scy,x1,y1,x2,y2 : integer);
  public
    angle : integer;
{    map   : TPic;}

    procedure DoMaprot; virtual;
    constructor create(aowner:tcomponent); override;
    destructor destroy; override;
    procedure LoadPic(fileName : string);
    procedure LoadBitmap(fileName : string);
    procedure SetBitmap(bm : TBitmap);
  published
    property Align;
    property Height default 200;
    property OnClick;
    property Width default 320;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
  end;

var
  map : TPic;
  FBitmap:TBitmapData;

procedure Register;

implementation

constructor tMaprot.create;
begin
  inherited create(aowner);
  makepalette;
  height:=200;
  width:=320;
//  fillchar(fbitmap,64000,15);
end;

destructor tMaprot.destroy;
begin
  killpalette;
  inherited destroy;
end;

procedure tMaprot.paint;
var
  dc:hdc;
  BitsInfo:pbitmapinfo;
  i:integer;
  oldpal:hpalette;
begin
  dc:=canvas.handle;
  oldpal:=selectpalette(dc,fpalette,false);
  getmem(bitsinfo,sizeof(tbitmapinfoheader)+256*2);
  with bitsinfo^ do
    begin
      with bmiheader do
        begin
          bisize:=40;
          biwidth:=320;
          biheight:=200;
          biplanes:=1;
          bibitcount:=8;
          bicompression:=0;
          bisizeimage:=64000;
          bixpelspermeter:=0;
          biypelspermeter:=0;
          biclrused:=0;
          biclrimportant:=0;
        end;

      for i:=0 to 127 do
        begin
          bmicolors[i].rgbblue:=2*i;
          bmicolors[i].rgbgreen:=0;
          bmicolors[i].rgbred:=2*i+1;
          bmicolors[i].rgbreserved:=0;
        end;

    end;

  realizePalette(dc);
//  setdibitstodevice(dc,0,0,width,height, 0,0,0,200,@FBitmap,Bitsinfo^,dib_Pal_Colors);
  stretchDIBits(dc,0,0,width,height,0,0,320,200,@FBitmap,BitsInfo^,dib_Pal_Colors,SRCCOPY);
  selectpalette(dc,oldpal,false);
  freemem(bitsinfo,sizeof(tbitmapinfoheader)+256*2);
end;

procedure tMaprot.makepalette;
var
  pal:plogpalette;
  i:integer;

  procedure setpalettereg(c,r,g,b:byte);
  begin
    pal^.palpalentry[c].pered:=r*4;
    pal^.palpalentry[c].pegreen:=g*2;
    pal^.palpalentry[c].peblue:=b;
    pal^.palpalentry[c].peflags:=0;
  end;

begin
  getmem(pal,4+256*4);
  fillchar(pal^,4+256*4,0);
  pal^.palVersion:=$300;
  pal^.palNumEntries:=256;
  for i:=0 to 255 do setpaletteReg(i,i,i,i);

  fpalette:=createpalette(pal^);
  freemem(pal,4+256*4);
end;

procedure tMaprot.killpalette;
begin
  deleteobject(fpalette);
end;

function tMaprot.getpalette;
begin
  getpalette:=fpalette;
end;

procedure TMapRot.MapLine(scy,x1,y1,x2,y2 : integer);
var
  qq,ps,ddx,ddy,sx,sy,x,y,a,b,m,n : integer;
  dc                              : byte;
  mapSeg                          : word;

begin
  ddx:=abs(x2-x1);
  ddy:=abs(y2-y1);

  if ddx>ddy then
    m:=ddx
  else
    m:=ddy;

  x:=x1;
  y:=y1;

  ps := x+y*128;
  qq := scy*320;
  a:=m shr 1;
  b:=m shr 1;

  if y2<y1 then
    sy:=-128
  else
    sy:=128;

  if x2>x1 then
    sx:=1
  else
    sx:=-1;

asm
        mov eax,a
        mov ebx,b
        mov esi,offset map
        add esi,ps
        mov edi,offset FBitmap
        add edi,qq

        mov ecx,0    { cx is counter }

@Next_dot:
        add eax,ddx
        cmp eax,m
        jnae @loc1

        add esi,sx
        sub eax,m

@loc1:
        add ebx,ddy
        cmp ebx,m
        jnae @loc2

        add esi,sy
        sub ebx,m

@loc2:
        sub esi,offset map
        and esi,$3fff
        add esi,offset map
        mov dl,[esi]
//        shr dl,4
        mov dh,dl
        mov [edi],dx
        inc edi
        inc edi
        inc ecx
        cmp ecx,160
        jne @Next_dot

@exit_loop:
  end;
end;

procedure tMaprot.DoMaprot;
const
  k=3.141/180;

var
  i,x1,y1,x2,y2 : integer;
  th,c,s        : real;

begin
  th:=angle*k;
  c:=cos(th); s:=sin(th);

  for i:=0 to 99 do
    begin
      x1:=round(-160*c-i*s);
      y1:=round(-160*s+i*c);
      x2:=round( 159*c-i*s);
      y2:=round( 159*s+i*c);
      MAPline(2*i  ,x1,y1,x2,y2);
      MAPline(2*i+1,x1,y1,x2,y2);
    end;

  paint;
end;

procedure TMapRot.LoadPic(fileName : string);
var
  infile : file of TPic;

begin
  assignFile(infile,fileName);
  reset(infile);
  read(infile,map);
  close(infile);
end;

procedure TMapRot.LoadBitmap(fileName : string);
var
  bm  : TBitmap;

begin
  bm:=TBitmap.Create;
  try
    bm.LoadFromFile(fileName);

    SetBitmap(bm);
  finally
    bm.free;
  end;
end;

procedure TMaprot.SetBitmap(bm : TBitmap);
var
  i,j : integer;

  function ColorOf(c : TColor):integer;
  begin
    if c=-1 then
      c:=0;

    Result:=GetNearestPaletteIndex(fPalette,c);
  end;

begin
  fPalette:=bm.Palette;

  for i:=0 to 127 do
    for j:=0 to 127 do
      map[i,j]:=ColorOf(bm.canvas.pixels[i,j]);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TMaprot]);
end;

end.
